// This file is part of GNOME Games. License: GPL-3.0+.

[GtkTemplate (ui = "/org/gnome/Games/ui/reset-controller-mapping-dialog.ui")]
public class Games.ResetControllerMappingDialog : Gtk.MessageDialog {
}
